@@ -1,11 +1,5 @@
-/* LibTomCrypt, modular cryptographic library -- Tom St Denis
- *
- * LibTomCrypt is a library that provides various cryptographic
- * algorithms in a highly modular and flexible manner.
- *
- * The library is free for all purposes without any express
- * guarantee it works.
- */
+/* LibTomCrypt, modular cryptographic library -- Tom St Denis */
+/* SPDX-License-Identifier: Unlicense */
 #include "tomcrypt_private.h"
 
 /**
@@ -90,6 +84,11 @@ const char *crypt_build_settings =
 #endif
 #if defined(LTC_DES)
    "   DES\n"
+   "   DES-X\n"
+   "   3DES\n"
+#endif
+#if defined(LTC_SM4)
+   "   SM4\n"
 #endif
 #if defined(LTC_CAST5)
    "   CAST5\n"
@@ -321,15 +320,21 @@ const char *crypt_build_settings =
 #if defined(LTC_FORTUNA)
     "   Fortuna (" NAME_VALUE(LTC_FORTUNA_POOLS) ", "
 #if defined(LTC_FORTUNA_RESEED_RATELIMIT_TIMED)
-    "LTC_FORTUNA_RESEED_RATELIMIT_TIMED, "
+    "LTC_FORTUNA_RESEED_RATELIMIT_TIMED"
 #else
     "LTC_FORTUNA_RESEED_RATELIMIT_STATIC, " NAME_VALUE(LTC_FORTUNA_WD)
+#endif
+#if defined(LTC_FORTUNA_USE_ENCRYPT_ONLY)
+    ", LTC_FORTUNA_USE_ENCRYPT_ONLY"
 #endif
     ")\n"
 #endif
 #if defined(LTC_SOBER128)
     "   SOBER128\n"
 #endif
+#if defined(LTC_WIN32_BCRYPT)
+    "   WIN32_BCRYPT\n"
+#endif
 
     "\nPK Crypto:\n"
 #if defined(LTC_MRSA)
@@ -422,6 +427,9 @@ const char *crypt_build_settings =
 #if defined(LTC_ADLER32)
     " ADLER32 "
 #endif
+#if defined(LTC_AES_NI)
+    " AES-NI "
+#endif
 #if defined(LTC_BASE64)
     " BASE64 "
 #endif
@@ -467,9 +475,17 @@ const char *crypt_build_settings =
     " PBES1 "
     " PBES2 "
 #endif
+#if defined(LTC_PEM)
+    " PEM "
+    " " NAME_VALUE(LTC_PEM_DECODE_BUFSZ) " "
+    " " NAME_VALUE(LTC_PEM_READ_BUFSIZE) " "
+#endif
 #if defined(LTC_SSH)
     " SSH "
 #endif
+#if defined(LTC_PEM_SSH)
+    " OpenSSH-PEM "
+#endif
 #if defined(LTC_DEVRANDOM)
     " LTC_DEVRANDOM "
 #endif
@@ -556,7 +572,3 @@ const char *crypt_build_settings =
     "\n"
     ;
 
-
-/* ref:         $Format:%D$ */
-/* git commit:  $Format:%H$ */
-/* commit time: $Format:%ai$ */
