@@ -1,11 +1,5 @@
-/* LibTomCrypt, modular cryptographic library -- Tom St Denis
- *
- * LibTomCrypt is a library that provides various cryptographic
- * algorithms in a highly modular and flexible manner.
- *
- * The library is free for all purposes without any express
- * guarantee it works.
- */
+/* LibTomCrypt, modular cryptographic library -- Tom St Denis */
+/* SPDX-License-Identifier: Unlicense */
 #include "tomcrypt_private.h"
 
 #ifdef LTC_DER
@@ -17,7 +11,7 @@
 int pk_oid_cmp_with_ulong(const char *o1, const unsigned long *o2, unsigned long o2size)
 {
    unsigned long i;
-   char tmp[256] = { 0 };
+   char tmp[LTC_OID_MAX_STRLEN] = { 0 };
    int err;
 
    if (o1 == NULL || o2 == NULL) return CRYPT_ERROR;
@@ -48,7 +42,3 @@ int pk_oid_cmp_with_asn1(const char *o1, const ltc_asn1_list *o2)
 }
 
 #endif
-
-/* ref:         $Format:%D$ */
-/* git commit:  $Format:%H$ */
-/* commit time: $Format:%ai$ */
